@@ -87,12 +87,12 @@ This is used by the import command.
 module Hledger.Read (
 
   -- * Journal files
-  PrefixedFilePath,
   defaultJournal,
-  defaultJournalWith,
   defaultJournalSafely,
-  defaultJournalSafelyWith,
+  defaultJournalWith,
+  defaultJournalWithSafely,
   defaultJournalPath,
+  defaultJournalPathSafely,
   requireJournalFileExists,
   ensureJournalFileExists,
   journalEnvVar,
@@ -132,8 +132,8 @@ module Hledger.Read (
 ) where
 
 --- ** imports
-import qualified Control.Exception as C
-import Control.Monad (unless, when, forM, (<=<))
+import Control.Exception qualified as C
+import Control.Monad (unless, when, forM, (>=>))
 import "mtl" Control.Monad.Except (ExceptT(..), runExceptT, liftEither)
 import Control.Monad.IO.Class (MonadIO, liftIO)
 import Data.Default (def)
@@ -144,8 +144,8 @@ import Data.Maybe (catMaybes, fromMaybe)
 import Data.Ord (comparing)
 import Data.Semigroup (sconcat)
 import Data.Text (Text)
-import qualified Data.Text as T
-import qualified Data.Text.IO as T
+import Data.Text qualified as T
+import Data.Text.IO qualified as T
 import Data.Time (Day)
 import Safe (headDef, headMay)
 import System.Directory (doesFileExist)
@@ -175,7 +175,6 @@ import Text.Printf (printf)
 --- ** journal reading
 
 journalEnvVar           = "LEDGER_FILE"
-journalEnvVar2          = "LEDGER"
 journalDefaultFilename  = ".hledger.journal"
 
 -- | Read the default journal file specified by the environment, 
@@ -183,53 +182,64 @@ journalDefaultFilename  = ".hledger.journal"
 defaultJournal :: IO Journal
 defaultJournal = defaultJournalSafely >>= either error' return -- PARTIAL:
 
--- | Read the default journal file specified by the environment,
--- with the given input options, or raise an error.
-defaultJournalWith :: InputOpts -> IO Journal
-defaultJournalWith iopts = defaultJournalSafelyWith iopts >>= either error' return -- PARTIAL:
-
--- | Read the default journal file specified by the environment,
--- with default input options, or return an error message.
+-- | Like defaultJournal, but return an error message instead of raising an error.
 defaultJournalSafely :: IO (Either String Journal)
-defaultJournalSafely = defaultJournalSafelyWith definputopts
+defaultJournalSafely = defaultJournalWithSafely definputopts
 
--- | Read the default journal file specified by the environment,
--- with the given input options, or return an error message.
-defaultJournalSafelyWith :: InputOpts -> IO (Either String Journal)
-defaultJournalSafelyWith iopts = (do
+-- | Like defaultJournal, but use the given input options.
+defaultJournalWith :: InputOpts -> IO Journal
+defaultJournalWith iopts = defaultJournalWithSafely iopts >>= either error' return -- PARTIAL:
+
+-- | Like defaultJournalWith, but return an error message instead of raising an error.
+defaultJournalWithSafely :: InputOpts -> IO (Either String Journal)
+defaultJournalWithSafely iopts = (do
   f <- defaultJournalPath
   runExceptT $ readJournalFile iopts f
-  ) `C.catches` [  -- XXX
+  )
+  `C.catches` [  -- XXX
      C.Handler (\(e :: C.ErrorCall)   -> return $ Left $ show e)
     ,C.Handler (\(e :: C.IOException) -> return $ Left $ show e)
     ]
--- | Get the default journal file path specified by the environment.
--- Like ledger, we look first for the LEDGER_FILE environment
--- variable, and if that does not exist, for the legacy LEDGER
--- environment variable. If neither is set, or the value is blank,
--- return the hard-coded default, which is @.hledger.journal@ in the
--- users's home directory (or in the current directory, if we cannot
--- determine a home directory).
+
+-- | Get the default journal file path, and check that it exists; or raise an error.
+--
+-- This looks for the LEDGER_FILE environment variable, like Ledger.
+-- The value should be a file path; ~ at the start is supported, meaning user's home directory.
+-- The value can also be a glob pattern, for convenience; if so we consider only the first matched file.
+-- If no such file exists, an error is raised.
+--
+-- If LEDGER_FILE is unset or set to the empty string, we return a default file path:
+-- @.hledger.journal@ in the user's home directory.
+-- Or if we can't find the user's home directory, in the current directory.
+-- If this default file doesn't exist, an error is raised.
+--
 defaultJournalPath :: IO String
 defaultJournalPath = do
-  p <- envJournalPath
-  if null p
-  then defpath
+  ledgerfile <- getEnv journalEnvVar `C.catch` (\(_::C.IOException) -> return "")
+  if null ledgerfile
+  then do
+    homedir <- fromMaybe "" <$> getHomeSafe
+    let defaultfile = homedir </> journalDefaultFilename
+    exists <- doesFileExist defaultfile
+    if exists then return defaultfile
+    -- else error' $ "LEDGER_FILE is unset and \"" <> defaultfile <> "\" was not found"
+    else error' $ "neither LEDGER_FILE nor \"" <> defaultfile <> "\" were found"
   else do
-    ps <- expandGlob "." p `C.catch` (\(_::C.IOException) -> return [])
-    maybe defpath return $ headMay ps
-    where
-      envJournalPath =
-        getEnv journalEnvVar
-         `C.catch` (\(_::C.IOException) -> getEnv journalEnvVar2
-                                            `C.catch` (\(_::C.IOException) -> return ""))
-      defpath = do
-        home <- fromMaybe "" <$> getHomeSafe
-        return $ home </> journalDefaultFilename
+    mf <- headMay <$> expandGlob "." ledgerfile `C.catch` (\(_::C.IOException) -> return [])
+    case mf of
+      Just f -> return f
+      Nothing -> error' $ "LEDGER_FILE points to nonexistent \"" <> ledgerfile <> "\""
 
--- | A file path optionally prefixed by a reader name and colon
--- (journal:, csv:, timedot:, etc.).
-type PrefixedFilePath = FilePath
+-- | Like defaultJournalPath, but return an error message instead of raising an error.
+defaultJournalPathSafely :: IO (Either String String)
+defaultJournalPathSafely = (do
+  f <- defaultJournalPath
+  return $ Right f
+  )
+  `C.catches` [
+     C.Handler (\(e :: C.ErrorCall)   -> return $ Left $ show e)
+    ,C.Handler (\(e :: C.IOException) -> return $ Left $ show e)
+    ]
 
 -- | @readJournal iopts mfile txt@
 --
@@ -337,9 +347,9 @@ readJournalFiles iopts@InputOpts{strict_, new_, new_save_} prefixedfiles = do
   when (new_ && new_save_) $ liftIO $ saveLatestDatesForFiles latestdatesforfiles
   return j
 
--- The implementation of readJournalFiles, but with --new, 
--- also returns the latest transaction date(s) read in each file.
--- Used by the import command, to save those at the end.
+-- The implementation of readJournalFiles.
+-- With --new, it also returns the latest transaction date(s) read in each file
+-- (used by the import command).
 readJournalFilesAndLatestDates :: InputOpts -> [PrefixedFilePath] -> ExceptT String IO (Journal, [LatestDatesForFile])
 readJournalFilesAndLatestDates iopts pfs = do
   (js, lastdates) <- unzip <$> mapM (readJournalFileAndLatestDates iopts) pfs
@@ -351,7 +361,7 @@ readJournal' = orDieTrying . readJournal definputopts Nothing
 
 -- | An even easier version of readJournal' which takes a 'Text' instead of a 'Handle'.
 readJournal'' :: Text -> IO Journal
-readJournal'' = readJournal' <=< inputToHandle
+readJournal'' = textToHandle >=> readJournal'
 
 -- | An easy version of 'readJournalFile' which assumes default options, and fails
 -- in the IO monad.
@@ -413,6 +423,7 @@ type LatestDates = [Day]
 
 -- The path of an input file, and its current "LatestDates".
 data LatestDatesForFile = LatestDatesForFile FilePath LatestDates
+  deriving Show
 
 -- | Get all instances of the latest date in an unsorted list of dates.
 -- Ie, if the latest date appears once, return it in a one-element list,
@@ -423,8 +434,10 @@ latestDates = {-# HLINT ignore "Avoid reverse" #-}
 
 -- | Save the given latest date(s) seen in the given data FILE,
 -- in a hidden file named .latest.FILE, creating it if needed.
+-- Unless no latest dates are provided, in which case do nothing.
 saveLatestDates :: LatestDates -> FilePath -> IO ()
-saveLatestDates dates f = T.writeFile (latestDatesFileFor f) $ T.unlines $ map showDate dates
+saveLatestDates dates f = when (not $ null dates) $
+  T.writeFile (latestDatesFileFor f) $ T.unlines $ map showDate dates
 
 -- | Save each file's latest dates.
 saveLatestDatesForFiles :: [LatestDatesForFile] -> IO ()
